using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;

namespace SNMP3
{
    public static class TopologyImage
    {
        private static StringFormat stred = new StringFormat();
        private static Font pismo = new Font("Arial", 7);

        public static void vytvor(ref List<PC> PCs, string vystup)
        {
            //zarovnani textu na stred
            stred.Alignment = StringAlignment.Center;

            //nactu seznam siti
            List<byte[]> site = vratSite(ref PCs);

            int sirka = PCs.Count * 150 + 40;
            int vyska = (site.Count + 1) * 200 + 40;

            //vytvorim novou bitmapu
            Bitmap myBitmap = new Bitmap(sirka, vyska);
            Graphics g = Graphics.FromImage(myBitmap);

            g.Clear(Color.White);//bile pozadi

            //vykrslim site pod sebe
            for (int y = 0; y < site.Count; y++)
            {
                g.FillRectangle(Brushes.Black, 20, (y + 1) * 200, PCs.Count * 140, 11);
                g.DrawString(site[y][0] + "." + site[y][1] + "." + site[y][2] + "." + site[y][3], pismo, Brushes.White, 20, (y + 1) * 200);
            }

            //projdu PC a vykreslim do prislusne site
            int x = 20;
            foreach (PC pc in PCs)
            {
                bool[] patri = new bool[site.Count];
                for (int i = 0; i < site.Count; i++)//zjistim, do jake site patri dany PC
                {
                    patri[i] = pc.patriDoSite(site[i]);
                }

                bool nakreslenPC = false;

                for (int i = 0; i < site.Count; i++)
                {
                    if (patri[i])
                    {
                        if (!nakreslenPC)
                        {
                            g.DrawImage(Resource1.PC, x, (i + 1) * 200 + 70);
                            g.DrawString(pc.hostname, pismo, Brushes.Gray, x + (Resource1.PC.Width / 2), (i + 1) * 200 + 70 + Resource1.PC.Height, stred);
                        }
                        g.DrawLine(Pens.Black, x + (Resource1.PC.Width / 2), (i + 1) * 200, x + (Resource1.PC.Width / 2), (i + 1) * 200 + (nakreslenPC ? -25 : 60));


                        //najdu prislusnou IP adresu v interfaces
                        foreach (iface ifc in pc.ifaces)
                        {
                            if (ifc.sit[0] == site[i][0] && ifc.sit[1] == site[i][1] && ifc.sit[2] == site[i][2] && ifc.sit[3] == site[i][3])
                            {
                                g.DrawString(ifc.adress[0] + "." + ifc.adress[1] + "." + ifc.adress[2] + "." + ifc.adress[3], pismo, Brushes.Gray, x + (Resource1.PC.Width / 2) + 5, (i + 1) * 200 + +(nakreslenPC ? -25 : 30));
                                g.DrawString(ifc.nazev.Substring(0, 18) + "...", pismo, Brushes.Gray, x + (Resource1.PC.Width / 2) + 5, (i + 1) * 200 + 9 + +(nakreslenPC ? -25 : 30));
                                break;
                            }
                        }
                        nakreslenPC = true;
                    }
                }
                x += Resource1.PC.Width;
            }

            myBitmap.Save(vystup);
        }

        private static List<byte[]> vratSite(ref List<PC> PCs)
        {
            List<byte[]> site = new List<byte[]>();

            foreach (PC pc in PCs)
            {
                foreach (iface i in pc.ifaces)
                {
                    // vynechm loopback rozhran a adresy 0.0.0.0
                    if (i.nazev.IndexOf("Loopback") < 0 &&
                        (i.sit[0] != 0 || i.sit[1] != 0 || i.sit[2] != 0 || i.sit[3] != 0))
                    {
                        bool obsahuje = false;

                        //pokud seznam s neobsahuje....
                        foreach (byte[] sit in site)
                        {
                            if (sit[0] == i.sit[0] && sit[1] == i.sit[1] && sit[2] == i.sit[2] && sit[3] == i.sit[3])
                            {
                                obsahuje = true;
                                break;
                            }
                        }
                        //... pridam ji do seznamu
                        if (!obsahuje)
                            site.Add(i.sit);
                    }
                }
            }
            return site;
        }
    }
}
