using System;
using System.Collections.Generic;
using System.Text;
using System.Net;
using System.Net.Sockets;
using System.IO;
using System.Threading;

namespace SNMP3
{
    public class SNMPRequest
    {
        private int _timeout;
        private string _community; //comunity string
        private Thread _thread; //vlakno obsluhy
        private Socket _sock; //socket
        private string _adress; //ip adresa
        private IPEndPoint _iep;
        private EndPoint _ep;

        public bool hotovo = false;

        public SNMPRequest(string adress, string community, int timeout)
        {
            this._adress = adress;
            this._community = community;
            this._timeout = timeout;

            _thread = new Thread(new ThreadStart(runThread));
            _thread.Start();
        }

        private void runThread()
        {
            //Send packet to destination
            this._sock = new Socket(AddressFamily.InterNetwork, SocketType.Dgram, ProtocolType.Udp);
            this._sock.SetSocketOption(SocketOptionLevel.Socket, SocketOptionName.ReceiveTimeout, _timeout);

            this._iep = new IPEndPoint(IPAddress.Parse(this._adress), 161);
            this._ep = (EndPoint)this._iep;

            Console.Write(scan());
            Console.Write("|\n");

            this._sock.Close();
            hotovo = true;
            this._thread.Abort();
        }

        private byte[] getInfo(string request, string mibstring)
        {
            byte[] packet = new byte[1024];
            byte[] mib = new byte[1024];
            int snmplen;
            int comlen = this._community.Length;
            string[] mibvals = mibstring.Split('.');
            int miblen = mibvals.Length;
            int cnt = 0, temp, i;
            int orgmiblen = miblen;
            int pos = 0;

            // Convert the string MIB into a byte array of integer values
            // Unfortunately, values over 128 require multiple bytes
            // which also increases the MIB length
            for (i = 0; i < orgmiblen; i++)
            {
                temp = Convert.ToInt16(mibvals[i]);
                if (temp > 127)
                {
                    mib[cnt] = Convert.ToByte(128 + (temp / 128));
                    mib[cnt + 1] = Convert.ToByte(temp - ((temp / 128) * 128));
                    cnt += 2;
                    miblen++;
                }
                else
                {
                    mib[cnt] = Convert.ToByte(temp);
                    cnt++;
                }
            }
            snmplen = 29 + comlen + miblen - 1;  //Length of entire SNMP packet

            //The SNMP sequence start
            packet[pos++] = 0x30; //Sequence start
            packet[pos++] = Convert.ToByte(snmplen - 2);  //sequence size

            //SNMP version
            packet[pos++] = 0x02; //Integer type
            packet[pos++] = 0x01; //length
            packet[pos++] = 0x00; //SNMP version 1

            //Community name
            packet[pos++] = 0x04; // String type
            packet[pos++] = Convert.ToByte(comlen); //length
            //Convert community name to byte array
            byte[] data = Encoding.ASCII.GetBytes(this._community);
            for (i = 0; i < data.Length; i++)
            {
                packet[pos++] = data[i];
            }

            //Add GetRequest or GetNextRequest value
            if (request == "get")
                packet[pos++] = 0xA0;
            else
                packet[pos++] = 0xA1;

            packet[pos++] = Convert.ToByte(20 + miblen - 1); //Size of total MIB

            //Request ID
            packet[pos++] = 0x02; //Integer type
            packet[pos++] = 0x04; //length
            packet[pos++] = 0x00; //SNMP request ID
            packet[pos++] = 0x00;
            packet[pos++] = 0x00;
            packet[pos++] = 0x01;

            //Error status
            packet[pos++] = 0x02; //Integer type
            packet[pos++] = 0x01; //length
            packet[pos++] = 0x00; //SNMP error status

            //Error index
            packet[pos++] = 0x02; //Integer type
            packet[pos++] = 0x01; //length
            packet[pos++] = 0x00; //SNMP error index

            //Start of variable bindings
            packet[pos++] = 0x30; //Start of variable bindings sequence

            packet[pos++] = Convert.ToByte(6 + miblen - 1); // Size of variable binding

            packet[pos++] = 0x30; //Start of first variable bindings sequence
            packet[pos++] = Convert.ToByte(6 + miblen - 1 - 2); // size
            packet[pos++] = 0x06; //Object type
            packet[pos++] = Convert.ToByte(miblen - 1); //length

            //Start of MIB
            packet[pos++] = 0x2b;
            //Place MIB array in packet
            for (i = 2; i < miblen; i++)
                packet[pos++] = Convert.ToByte(mib[i]);
            packet[pos++] = 0x05; //Null object value
            packet[pos++] = 0x00; //Null



            //Receive response from packet
            try
            {
                this._sock.SendTo(packet, snmplen, SocketFlags.None, this._iep);
                int recv = this._sock.ReceiveFrom(packet, ref this._ep);
            }
            catch (SocketException)
            {
                packet[0] = 0xff;
            }
            return packet;
        }

        private string getnextMIB(byte[] mibin)
        {
            string output = "1.3";
            int commlength = mibin[6];
            int mibstart = 6 + commlength + 17; //find the start of the mib section
            //The MIB length is the length defined in the SNMP packet
            // minus 1 to remove the ending .0, which is not used
            int miblength = mibin[mibstart] - 1;
            mibstart += 2; //skip over the length and 0x2b values
            int mibvalue;

            for (int i = mibstart; i < mibstart + miblength; i++)
            {
                mibvalue = Convert.ToInt16(mibin[i]);
                if (mibvalue > 128)
                {
                    mibvalue = (mibvalue / 128) * 128 + Convert.ToInt16(mibin[i + 1]);
                    i++;
                }
                output += "." + mibvalue;
            }
            return output;
        }


        public string scan()
        {
            StringBuilder output = new StringBuilder();
            int commlength, miblength, datatype, datalength, datastart;
            byte[] response = new byte[1024];

            output.Append("|+ " + this._adress + "\n");

            // Send sysName SNMP request
            response = this.getInfo("get", "1.3.6.1.2.1.1.5.0");
            if (response[0] == 0xff)
                return "|+ " + this._adress + "\n|- !! Host " + this._adress + " je nedostupn !!\n";

            commlength = Convert.ToInt16(response[6]);
            miblength = Convert.ToInt16(response[23 + commlength]);

            // Extract the MIB data from the SNMP response
            datatype = Convert.ToInt16(response[24 + commlength + miblength]);
            datalength = Convert.ToInt16(response[25 + commlength + miblength]);
            datastart = 26 + commlength + miblength;
            string data = Encoding.ASCII.GetString(response, datastart, datalength);
            output.Append("||- sysname: " + data + "\n");

            PC thisPC = null;

            foreach (PC pc in SNMPInfo.PCs)
            {
                if (pc.hostname == data)
                {
                    return "";
                }
            }

            if (thisPC == null)
            {
                thisPC = new PC();
                SNMPInfo.PCs.Add(thisPC);
                thisPC.hostname = data;
            }


            //hw/sw info
            response = this.getInfo("get", "1.3.6.1.2.1.1.1.0");
            if (response[0] == 0xff)
                return "|+ " + this._adress + "\n|- !! Host " + this._adress + " je nedostupn !!\n";

            // If response, get the community name and MIB lengths
            commlength = Convert.ToInt16(response[6]);
            miblength = Convert.ToInt16(response[23 + commlength]);

            // Extract the MIB data from the SNMP response
            datatype = Convert.ToInt16(response[24 + commlength + miblength]);
            datalength = Convert.ToInt16(response[43]);
            datastart = 44;
            thisPC.sysid = Encoding.ASCII.GetString(response, datastart, datalength);
            output.Append("||- sysid: " + thisPC.sysid + "\n");



            // sysLocation SNMP request
            response = getInfo("get", "1.3.6.1.2.1.1.6.0");
            if (response[0] == 0xff)
            {
                return "|- !! Host " + this._adress + " je nedostupn !!\n";
            }

            // If response, get the community name and MIB lengths
            commlength = Convert.ToInt16(response[6]);
            miblength = Convert.ToInt16(response[23 + commlength]);

            // Extract the MIB data from the SNMP response
            datatype = Convert.ToInt16(response[24 + commlength + miblength]);
            datalength = Convert.ToInt16(response[25 + commlength + miblength]);
            datastart = 26 + commlength + miblength;
            thisPC.location = Encoding.ASCII.GetString(response, datastart, datalength);
            output.Append("||- umstn: " + thisPC.location + "\n");


            List<iface> ifaces = new List<iface>();

            //list all IP
            response = getInfo("get next", "1.3.6.1.2.1.4.20");
            while (true)
            {
                // Get the community and MIB lengths of the response
                commlength = Convert.ToInt16(response[6]);
                miblength = Convert.ToInt16(response[23 + commlength]);

                // Extract the MIB data from the SNMp response
                datatype = Convert.ToInt16(response[24 + commlength + miblength]);
                datalength = Convert.ToInt16(response[25 + commlength + miblength]);
                datastart = 26 + commlength + miblength;
                if (datatype == 0x40)
                {
                    iface ifc = new iface();
                    for (int i = 0; i < 4; i++)
                    {
                        ifc.adress[i] = response[datastart + i];
                    }
                    //= response[datastart] + "." + response[datastart + 1] + "." + response[datastart + 2] + "." + response[datastart + 3];
                    ifaces.Add(ifc);
                }
                else
                    break;

                response = getInfo("get next", getnextMIB(response));
            }

            //mask&adress of ifaces
            for (int i = 0; i < ifaces.Count; i++)
            {
                response = getInfo("get", "1.3.6.1.2.1.4.20.1.2." + ifaces[i].adress[0] + "." + ifaces[i].adress[1] + "." + ifaces[i].adress[2] + "." + ifaces[i].adress[3]);
                commlength = Convert.ToInt16(response[6]);
                miblength = Convert.ToInt16(response[23 + commlength]);
                datatype = Convert.ToInt16(response[24 + commlength + miblength]);
                datalength = Convert.ToInt16(response[25 + commlength + miblength]);
                datastart = 26 + commlength + miblength;
                ifaces[i].id = response[datastart];

                response = getInfo("get", "1.3.6.1.2.1.4.20.1.3." + ifaces[i].adress[0] + "." + ifaces[i].adress[1] + "." + ifaces[i].adress[2] + "." + ifaces[i].adress[3]);
                commlength = Convert.ToInt16(response[6]);
                miblength = Convert.ToInt16(response[23 + commlength]);
                datatype = Convert.ToInt16(response[24 + commlength + miblength]);
                datalength = Convert.ToInt16(response[25 + commlength + miblength]);
                datastart = 26 + commlength + miblength;
                //ifaces[i].mask = response[datastart] + "." + response[datastart + 1] + "." + response[datastart + 2] + "." + response[datastart + 3];
                for (int x = 0; x < 4; x++)
                {
                    ifaces[i].mask[x] = response[datastart + x];
                }

                for (int x = 0; x < 4; x++)
                {
                    ifaces[i].sit[x] = (byte)(ifaces[i].adress[x] & ifaces[i].mask[x]);
                }
            }

            thisPC.ifaces = ifaces;


            //projdu interfaceTables
            for (int i = 0; i < ifaces.Count; i++)
            {
                response = getInfo("get", "1.3.6.1.2.1.2.2.1.2." + ifaces[i].id);
                if (response[0] == 0xff)
                    return "|+ " + this._adress + "\n|- !! Host " + this._adress + " je nedostupn !!\n";

                // Get the community and MIB lengths of the response
                commlength = Convert.ToInt16(response[6]);
                miblength = Convert.ToInt16(response[23 + commlength]);

                // Extract the MIB data from the SNMp response
                datatype = Convert.ToInt16(response[24 + commlength + miblength]);
                datalength = Convert.ToInt16(response[25 + commlength + miblength]);
                datastart = 26 + commlength + miblength;

                //prisla odpoved
                if (datalength > 0)
                {
                    ifaces[i].nazev = Encoding.ASCII.GetString(response, datastart, datalength);
                }
            }

            foreach (iface i in thisPC.ifaces)
            {
                output.Append("||" + "+" + " rozhran: " + i.nazev + "\n");
                output.Append("|||- " + "IP: " + i.adress[0] + "." + i.adress[1] + "." + i.adress[2] + "." + i.adress[3] + "\n|||- " + "Maska: " + i.mask[0] + "." + i.mask[1] + "." + i.mask[2] + "." + i.mask[3] + "\n");
                output.Append("|||- " + "S: " + i.sit[0] + "." + i.sit[1] + "." + i.sit[2] + "." + i.sit[3] + "\n");
            }

            return output.ToString();
        }
    }

    public static class SNMPInfo
    {
        public static List<PC> PCs = new List<PC>();

        public static void enumInterfaces(string hostsFile, string cs, int timeout , string vystup)
        {

            //try
            {
                //nactu seznam hostu ze souboru
                string[] hosts = File.ReadAllLines(hostsFile);

                //vpis nastaven
                Console.WriteLine("Nastaven:");
                Console.WriteLine("Comunity string: " + cs);
                Console.WriteLine("Timeout: " + timeout + "ms\n");
                Console.WriteLine("Poet scanovanch host:" + hosts.Length);

                List<SNMPRequest> pozadavky = new List<SNMPRequest>();

                //projdu hosty
                foreach (string host in hosts)
                {
                    //zisk info pres SNMP z hosta
                    pozadavky.Add(new SNMPRequest(host, cs, timeout));
                }

                //spm po dobu timeoutu
                //Thread.Sleep(timeout);

                //oven dokonceni vsceh scanu
                bool hotovo = false;
                while (!hotovo)
                {
                    Thread.Sleep(500);
                    hotovo = true;
                    foreach (SNMPRequest p in pozadavky)
                    {
                        if (!p.hotovo)
                        {
                            hotovo = false;
                            break;
                        }
                    }
                }
                Console.WriteLine("\nScanovn hotovo!\nProbh generovn obrzku...\n");
                TopologyImage.vytvor(ref PCs, vystup);
                Console.WriteLine("Hotovo!");
            }
            //catch (Exception ex)
            //{
            //    Console.WriteLine("Chyba pi ten seznamu host!\n" + ex.Message);
            //}
        }
    }

    public class PC
    {
        public string hostname;
        public string location;
        public string sysid;

        public List<iface> ifaces = new List<iface>();

        public bool patriDoSite(byte[] sit)
        {
            foreach (iface i in this.ifaces)
            {
                if (i.sit[0] == sit[0]&&i.sit[1] == sit[1]&&i.sit[2] == sit[2]&&i.sit[3] == sit[3])
                    return true;
            }
            return false;
        }
    }

    public class iface
    {
        public int id;
        public string nazev;
        public byte[] adress = new byte[4];
        public byte[] mask = new byte[4];
        public byte[] sit = new byte[4];

        
    }
}
